
#ifndef __FSOTRANSPORT_H__
#define __FSOTRANSPORT_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <fsobasics.h>

G_BEGIN_DECLS


#define FSO_FRAMEWORK_TYPE_TRANSPORT_STATE (fso_framework_transport_state_get_type ())

#define FSO_FRAMEWORK_TYPE_TRANSPORT (fso_framework_transport_get_type ())
#define FSO_FRAMEWORK_TRANSPORT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FSO_FRAMEWORK_TYPE_TRANSPORT, FsoFrameworkTransport))
#define FSO_FRAMEWORK_TRANSPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FSO_FRAMEWORK_TYPE_TRANSPORT, FsoFrameworkTransportClass))
#define FSO_FRAMEWORK_IS_TRANSPORT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FSO_FRAMEWORK_TYPE_TRANSPORT))
#define FSO_FRAMEWORK_IS_TRANSPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FSO_FRAMEWORK_TYPE_TRANSPORT))
#define FSO_FRAMEWORK_TRANSPORT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FSO_FRAMEWORK_TYPE_TRANSPORT, FsoFrameworkTransportClass))

typedef struct _FsoFrameworkTransport FsoFrameworkTransport;
typedef struct _FsoFrameworkTransportClass FsoFrameworkTransportClass;
typedef struct _FsoFrameworkTransportPrivate FsoFrameworkTransportPrivate;

#define FSO_FRAMEWORK_TYPE_BASE_TRANSPORT (fso_framework_base_transport_get_type ())
#define FSO_FRAMEWORK_BASE_TRANSPORT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FSO_FRAMEWORK_TYPE_BASE_TRANSPORT, FsoFrameworkBaseTransport))
#define FSO_FRAMEWORK_BASE_TRANSPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FSO_FRAMEWORK_TYPE_BASE_TRANSPORT, FsoFrameworkBaseTransportClass))
#define FSO_FRAMEWORK_IS_BASE_TRANSPORT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FSO_FRAMEWORK_TYPE_BASE_TRANSPORT))
#define FSO_FRAMEWORK_IS_BASE_TRANSPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FSO_FRAMEWORK_TYPE_BASE_TRANSPORT))
#define FSO_FRAMEWORK_BASE_TRANSPORT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FSO_FRAMEWORK_TYPE_BASE_TRANSPORT, FsoFrameworkBaseTransportClass))

typedef struct _FsoFrameworkBaseTransport FsoFrameworkBaseTransport;
typedef struct _FsoFrameworkBaseTransportClass FsoFrameworkBaseTransportClass;
typedef struct _FsoFrameworkBaseTransportPrivate FsoFrameworkBaseTransportPrivate;

#define FSO_FRAMEWORK_TYPE_SERIAL_TRANSPORT (fso_framework_serial_transport_get_type ())
#define FSO_FRAMEWORK_SERIAL_TRANSPORT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FSO_FRAMEWORK_TYPE_SERIAL_TRANSPORT, FsoFrameworkSerialTransport))
#define FSO_FRAMEWORK_SERIAL_TRANSPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FSO_FRAMEWORK_TYPE_SERIAL_TRANSPORT, FsoFrameworkSerialTransportClass))
#define FSO_FRAMEWORK_IS_SERIAL_TRANSPORT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FSO_FRAMEWORK_TYPE_SERIAL_TRANSPORT))
#define FSO_FRAMEWORK_IS_SERIAL_TRANSPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FSO_FRAMEWORK_TYPE_SERIAL_TRANSPORT))
#define FSO_FRAMEWORK_SERIAL_TRANSPORT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FSO_FRAMEWORK_TYPE_SERIAL_TRANSPORT, FsoFrameworkSerialTransportClass))

typedef struct _FsoFrameworkSerialTransport FsoFrameworkSerialTransport;
typedef struct _FsoFrameworkSerialTransportClass FsoFrameworkSerialTransportClass;
typedef struct _FsoFrameworkSerialTransportPrivate FsoFrameworkSerialTransportPrivate;

#define FSO_FRAMEWORK_TYPE_PTY_TRANSPORT (fso_framework_pty_transport_get_type ())
#define FSO_FRAMEWORK_PTY_TRANSPORT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FSO_FRAMEWORK_TYPE_PTY_TRANSPORT, FsoFrameworkPtyTransport))
#define FSO_FRAMEWORK_PTY_TRANSPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FSO_FRAMEWORK_TYPE_PTY_TRANSPORT, FsoFrameworkPtyTransportClass))
#define FSO_FRAMEWORK_IS_PTY_TRANSPORT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FSO_FRAMEWORK_TYPE_PTY_TRANSPORT))
#define FSO_FRAMEWORK_IS_PTY_TRANSPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FSO_FRAMEWORK_TYPE_PTY_TRANSPORT))
#define FSO_FRAMEWORK_PTY_TRANSPORT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FSO_FRAMEWORK_TYPE_PTY_TRANSPORT, FsoFrameworkPtyTransportClass))

typedef struct _FsoFrameworkPtyTransport FsoFrameworkPtyTransport;
typedef struct _FsoFrameworkPtyTransportClass FsoFrameworkPtyTransportClass;
typedef struct _FsoFrameworkPtyTransportPrivate FsoFrameworkPtyTransportPrivate;

/*===========================================================================*/
typedef enum  {
	FSO_FRAMEWORK_TRANSPORT_ERROR_UNABLE_TO_OPEN,
	FSO_FRAMEWORK_TRANSPORT_ERROR_UNABLE_TO_WRITE
} FsoFrameworkTransportError;
#define FSO_FRAMEWORK_TRANSPORT_ERROR fso_framework_transport_error_quark ()
/*===========================================================================*/
typedef enum  {
	FSO_FRAMEWORK_TRANSPORT_STATE_CLOSED,
	FSO_FRAMEWORK_TRANSPORT_STATE_OPEN,
	FSO_FRAMEWORK_TRANSPORT_STATE_ALIVE,
	FSO_FRAMEWORK_TRANSPORT_STATE_FROZEN,
	FSO_FRAMEWORK_TRANSPORT_STATE_DEAD
} FsoFrameworkTransportState;

/*===========================================================================*/
typedef void (*FsoFrameworkTransportReadFunc) (FsoFrameworkTransport* transport, void* user_data);
typedef void (*FsoFrameworkTransportHupFunc) (FsoFrameworkTransport* transport, void* user_data);
/*===========================================================================*/
struct _FsoFrameworkTransport {
	GObject parent_instance;
	FsoFrameworkTransportPrivate * priv;
};

struct _FsoFrameworkTransportClass {
	GObjectClass parent_class;
	gboolean (*isOpen) (FsoFrameworkTransport* self);
	gboolean (*open) (FsoFrameworkTransport* self);
	char* (*getName) (FsoFrameworkTransport* self);
	void (*setDelegates) (FsoFrameworkTransport* self, FsoFrameworkTransportReadFunc readfunc, void* readfunc_target, FsoFrameworkTransportHupFunc hupfunc, void* hupfunc_target);
	void (*getDelegates) (FsoFrameworkTransport* self, FsoFrameworkTransportReadFunc* readfun, void** readfun_target, GDestroyNotify readfun_target_destroy_notify, FsoFrameworkTransportHupFunc* hupfun, void** hupfun_target, GDestroyNotify hupfun_target_destroy_notify);
	void (*setPriorities) (FsoFrameworkTransport* self, gint rp, gint wp);
	gint (*read) (FsoFrameworkTransport* self, void* data, gint len);
	void (*freeze) (FsoFrameworkTransport* self);
	void (*thaw) (FsoFrameworkTransport* self);
	gint (*write) (FsoFrameworkTransport* self, void* data, gint length);
	void (*close) (FsoFrameworkTransport* self);
};

/*===========================================================================*/
struct _FsoFrameworkBaseTransport {
	FsoFrameworkTransport parent_instance;
	FsoFrameworkBaseTransportPrivate * priv;
	char* name;
	guint speed;
	gboolean raw;
	gboolean hard;
	gint fd;
	FsoFrameworkTransportHupFunc hupfunc;
	gpointer hupfunc_target;
	FsoFrameworkTransportReadFunc readfunc;
	gpointer readfunc_target;
	GByteArray* buffer;
	FsoFrameworkLogger* logger;
};

struct _FsoFrameworkBaseTransportClass {
	FsoFrameworkTransportClass parent_class;
	char* (*repr) (FsoFrameworkBaseTransport* self);
};

/*===========================================================================*/
struct _FsoFrameworkSerialTransport {
	FsoFrameworkBaseTransport parent_instance;
	FsoFrameworkSerialTransportPrivate * priv;
};

struct _FsoFrameworkSerialTransportClass {
	FsoFrameworkBaseTransportClass parent_class;
};

/*===========================================================================*/
struct _FsoFrameworkPtyTransport {
	FsoFrameworkBaseTransport parent_instance;
	FsoFrameworkPtyTransportPrivate * priv;
};

struct _FsoFrameworkPtyTransportClass {
	FsoFrameworkBaseTransportClass parent_class;
};


GQuark fso_framework_transport_error_quark (void);
GType fso_framework_transport_state_get_type (void);
GType fso_framework_transport_get_type (void);
gboolean fso_framework_transport_isOpen (FsoFrameworkTransport* self);
gboolean fso_framework_transport_open (FsoFrameworkTransport* self);
char* fso_framework_transport_getName (FsoFrameworkTransport* self);
void fso_framework_transport_setDelegates (FsoFrameworkTransport* self, FsoFrameworkTransportReadFunc readfunc, void* readfunc_target, FsoFrameworkTransportHupFunc hupfunc, void* hupfunc_target);
void fso_framework_transport_getDelegates (FsoFrameworkTransport* self, FsoFrameworkTransportReadFunc* readfun, void** readfun_target, GDestroyNotify readfun_target_destroy_notify, FsoFrameworkTransportHupFunc* hupfun, void** hupfun_target, GDestroyNotify hupfun_target_destroy_notify);
void fso_framework_transport_setPriorities (FsoFrameworkTransport* self, gint rp, gint wp);
gint fso_framework_transport_read (FsoFrameworkTransport* self, void* data, gint len);
void fso_framework_transport_freeze (FsoFrameworkTransport* self);
void fso_framework_transport_thaw (FsoFrameworkTransport* self);
gint fso_framework_transport_write (FsoFrameworkTransport* self, void* data, gint length);
void fso_framework_transport_close (FsoFrameworkTransport* self);
FsoFrameworkTransport* fso_framework_transport_create (const char* type, const char* name, guint speed, gboolean raw, gboolean hard);
GType fso_framework_base_transport_get_type (void);
void fso_framework_base_transport_restartWriter (FsoFrameworkBaseTransport* self);
FsoFrameworkBaseTransport* fso_framework_base_transport_new (const char* name, guint speed, gboolean raw, gboolean hard);
FsoFrameworkBaseTransport* fso_framework_base_transport_construct (GType object_type, const char* name, guint speed, gboolean raw, gboolean hard);
char* fso_framework_base_transport_repr (FsoFrameworkBaseTransport* self);
gboolean fso_framework_base_transport_actionCallback (FsoFrameworkBaseTransport* self, GIOChannel* source, GIOCondition condition);
gboolean fso_framework_base_transport_writeCallback (FsoFrameworkBaseTransport* self, GIOChannel* source, GIOCondition condition);
GType fso_framework_serial_transport_get_type (void);
FsoFrameworkSerialTransport* fso_framework_serial_transport_new (const char* portname, guint portspeed, gboolean raw, gboolean hard);
FsoFrameworkSerialTransport* fso_framework_serial_transport_construct (GType object_type, const char* portname, guint portspeed, gboolean raw, gboolean hard);
GType fso_framework_pty_transport_get_type (void);
FsoFrameworkPtyTransport* fso_framework_pty_transport_new (void);
FsoFrameworkPtyTransport* fso_framework_pty_transport_construct (GType object_type);


G_END_DECLS

#endif
